#include "crypto.h"
#ifndef _cccam_
#define _cccam_

#ifndef BIGE
#define SCT_LEN(sct) (3+(((sct)[1]&0x0f)<<8)+(sct)[2])
#define SETCMDLEN(h,l) BYTE2_BE(&(h)->cmdlen,(l)-4)
#define CMDLEN(h)      (UINT16_BE(&(h)->cmdlen)+4)
#else
#warning Compiling for Little Endian
#define SCT_LEN(sct) (3+(((sct)[1]&0x0f)<<8)+(sct)[2])
#define SETCMDLEN(h,l) BYTE2_LE(&(h)->cmdlen,(l)-4)
#define CMDLEN(h)      (UINT16_LE(&(h)->cmdlen)+4)
#endif


// -- CCcam protocol structs ---------------------------------------------------

struct eshares {
  unsigned int shareid;  // BE
  unsigned short caid;   // BE
  unsigned char uphops, maxdown;
  unsigned char provcnt;
  unsigned int  provid[20];
  bool status;	// active or not
  bool deleted; // deleted or not.
};

struct cEcmInfo {
  unsigned short caId;
  unsigned int provId;
  unsigned short prgId; 
  unsigned short ecm_pid;
};

struct CmdHeader {
  unsigned char flags, cmd;
  unsigned short cmdlen; // BE
  } __attribute__((packed));


struct GenericCmd {
  struct CmdHeader header;
  unsigned char payload[0];
  } __attribute__((packed));

struct ClientInfo {
  struct CmdHeader header;
  char username[20];
  unsigned char nodeid[8];
  unsigned char wantemus;
  char version[32], build[32];
  } __attribute__((packed));

struct ServerInfo {
  struct CmdHeader header;
  unsigned char nodeid[8];
  char version[32], build[32];
  } __attribute__((packed));

struct EcmRequest {
  struct CmdHeader header;
  unsigned short caid;  // BE
  unsigned int provid;  // BE
  unsigned int shareid; // BE
  unsigned short sid;   // BE
  unsigned char datalen;
  unsigned char data[0];
  } __attribute__((packed));

struct EmmRequest {
  struct CmdHeader header;
  unsigned short caid;  // BE
  unsigned char dummy;  //XXX what is that?
  unsigned int provid;  // BE
  unsigned int shareid; // BE
  unsigned char datalen;
  unsigned char data[0];
  } __attribute__((packed));

struct DcwAnswer {
  struct CmdHeader header;
  unsigned char cw[16];
  } __attribute__((packed));

struct DelShare {
  struct CmdHeader header;
  unsigned int shareid;  // BE
  } __attribute__((packed));

struct AddShare {
  struct CmdHeader header;
  unsigned int shareid;  // BE
  unsigned int remoteid; // BE
  unsigned short caid;   // BE
  unsigned char uphops, maxdown;
  unsigned char cardserial[8];
  } __attribute__((packed));

struct ProvInfo {
  unsigned char count;
  struct {
    unsigned char provid[3];
    unsigned char sa[4];
    } prov[0];
  } __attribute__((packed));

struct NodeInfo {
  unsigned char count;
  unsigned char nodeid[8];
  } __attribute__((packed));

// -- cCardClientCCcam2 ---------------------------------------------------------

#define MAX_ECM_TIME 3000     // ms
#define PING_TIME    120*1000 // ms
#define STDLAG 1000
#define MAXLAG 5000

#define CMD05_MODE_UNKNOWN 0
#define CMD05_MODE_PLAIN   1
#define CMD05_MODE_AES     2
#define CMD05_MODE_CC      3
#define CMD05_MODE_RC4     4
#define CMD05_MODE_LEN0    5

class cCardClientCCcam2 {
friend void* proxy_function(void*);
private:
  int fd; //file descriptor for network connections.
  cCCcamCrypt encr, decr;
  unsigned char nodeid[8];
  int shareid1;
  char username[21], password[64], versstr[32], buildstr[32];
  char hostname[200];
  int  port,Hop;
  bool login, wantEmus;
  int pendingDCW,maxecmcount;
  int cmd05mode, cmd05off;
  unsigned char cmd05aes[16], cmd0Baes[16];
  cCCcamCrypt cmd05crypt;
  cCmd0cCrypt cmd0Ccrypt;
  //
  pthread_t myaction;
  pthread_mutex_t mtxshare;
  pthread_mutex_t mtxcw;
  pthread_mutex_t mtxsend;
  pthread_mutex_t mtxrecv;
  bool tamam;   //  cwwait timeuot icin
  eshares  ccshare[500];
  int shrcnt; // share count;
 
 
  void AddS(AddShare *share1, ProvInfo *prov1);
  void DelS(DelShare *oldshare);
  void PacketAnalyzer(const struct CmdHeader *hdr, int length);
  int  CryptRecv(unsigned char *data, int len, int to=-1);
  int  RecvMsg(unsigned char *buffer1, int length, int override_timeout);
  bool CryptSend(const unsigned char *data, int len);
  bool SendMsg(unsigned char *buffer1,  int length);
//protected:
  void Action(void);
public:
  bool ready;
  bool newcw;
  int dbg;
  int network_timeout;
  unsigned char cw[16];
  cCardClientCCcam2(char *server_url, unsigned int timeout,int hop);
  ~cCardClientCCcam2(void);
  bool Login(void);
  int HasECM(cEcmInfo *ecm, int istart);
  bool ProcessECM(cEcmInfo *ecm,unsigned char *data, unsigned char *Cw);
  };
#endif
